package com.persagy.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.entity.*;
import com.persagy.util.*;
import com.persagy.wsentity.SBuilding;
import com.persagy.wsentity.SSurvey;
import com.persagy.wsentity.SSurveyResult;
import com.persagy.wsentity2.TDefaultFunctionId;
import com.persagy.wsentity2.TMeter;
import com.persagy.wsentity2.TSurvey;
import com.persagy.wsentity2.TSurveyResult;
import com.sagacloud.util.compute_engine.core.CoreComputeUtil;
import com.sagacloud.util.compute_engine.core.CoreComputeUtilUtil;
import com.sagacloud.util.compute_engine.core.CoreInstance;
import com.sagacloud.util.compute_engine.core.CoreOutput;
import com.sagacloud.util.compute_engine.entity.Entity_ci_instance;
import com.sagacloud.util.compute_engine.entity.Entity_ci_instances;
import com.sagacloud.util.compute_engine.util.ConstantCE;
import com.sagacloud.util.compute_engine.util.InstanceUtil;
import com.sagacloud.util.compute_engine.util.QueryUtil;
import lombok.extern.slf4j.Slf4j;
import org.zillion.util.json.FastJsonCompareUtil;
import org.zillion.util.json.FastJsonUtil;

import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
public class ServiceUtil {
    public static synchronized Entity_ci_instances ci_instance_GetAll() throws Exception {
        try {
            Entity_ci_instances result = InstanceUtil.GetAll(ConstantCE.database_name);
            return result;
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            throw e;
        }
    }

    public static synchronized Entity_ci_instance ci_instance_Get(String object_sign, String instance_name) throws Exception {
        try {
            Entity_ci_instance result = InstanceUtil.Get(ConstantCE.database_name, object_sign, instance_name);
            return result;
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            throw e;
        }
    }

    public static synchronized void ci_instance_Request(String object_sign, String instance_name, String status_request) throws Exception {
        try {
            InstanceUtil.Request(ConstantCE.database_name, object_sign, instance_name, status_request);
            SyncUtil.Sync(object_sign, instance_name);
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            throw e;
        }
    }

    public static synchronized void ci_instance_interval(String object_sign, String instance_name, long interval) throws Exception {
        try {
            JSONObject Criteria = new JSONObject();
            Criteria.put("object_sign", object_sign);
            Criteria.put("instance_name", instance_name);
            JSONObject Set = new JSONObject();
            Set.put("compute_interval_seconds", interval);
            QueryUtil.update(ConstantCE.database_name, "ci_instance", Criteria, Set);
            SyncUtil.Sync(object_sign, instance_name);
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            throw e;
        }
    }

    public static synchronized void ci_instance_Update(Entity_ci_instance instance) throws Exception {
        try {
            JSONObject json = (JSONObject) FastJsonUtil.To_JSON(instance);
            Entity_ci_instance instance_exist = InstanceUtil.Get(ConstantCE.database_name, instance.getObject_sign(),
                    instance.getInstance_name());
            if (instance_exist != null) {
                JSONObject json_exist = (JSONObject) FastJsonUtil.To_JSON(instance_exist);
                if (FastJsonCompareUtil.Instance().Compare(json_exist, json)) {
                    // log.info("There is an instance equal to:\n" + json.toFormatString());
                    return;
                }
            }
            // log.info("Update instance:\n" + json.toFormatString());
            InstanceUtil.Delete(ConstantCE.database_name, instance.getObject_sign(), instance.getInstance_name());
            InstanceUtil.Insert_status(ConstantCE.database_name, instance,
                    ConstantWeb.name2status_request.get(instance.getInstance_name()));
            InstanceUtil.Insert(ConstantCE.database_name, instance);
            SyncUtil.Sync(instance.getObject_sign(), instance.getInstance_name());
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            throw e;
        }
    }

    public static synchronized void ci_instance_Delete(String object_sign, String instance_name) throws Exception {
        try {
            InstanceUtil.Delete(ConstantCE.database_name, object_sign, instance_name);
            SyncUtil.Sync(object_sign, instance_name);
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            throw e;
        }
    }

    public static synchronized void Survey_Delete(SSurvey survey) throws Exception {
        try {
            String instance_name = "EMS";
            for (SBuilding building : survey.getBuildingArray()) {
                String object_sign = building.getBuildingSign();
                ci_instance_Delete(object_sign, instance_name);
            }
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            throw e;
        }
    }

    public static synchronized SSurveyResult Survey_Update(SSurvey survey) throws Exception {
        try {
            SSurveyResult result = SurveyUtil.Check(survey);

            String projectSign_ = survey.getProjectSign().substring("Pj".length());
            String sign_new = projectSign_;
            if (ConstantWeb.BuildingUploadMap.size() > 0) {
                if (ConstantWeb.BuildingUploadMap.containsKey(projectSign_)) {
                    sign_new = ConstantWeb.BuildingUploadMap.get(projectSign_);
                } else {
                    return result;
                }
            }
            survey.setProjectSign("Pj" + sign_new);
            for (SBuilding building : survey.getBuildingArray()) {
                String buildingSign = building.getBuildingSign();
                if (buildingSign.startsWith("Bd")) {
                    String buildingSign_ = buildingSign.substring("Bd".length());
                    if (buildingSign_.startsWith(projectSign_)) {
                        String buildingSign_new = "Bd" + sign_new + buildingSign_.substring(projectSign_.length());
                        building.setBuildingSign(buildingSign_new);
                    }
                } else if (buildingSign.startsWith("VOBd")) {
                    String buildingSign_ = buildingSign.substring("VOBd".length());
                    if (buildingSign_.startsWith(projectSign_)) {
                        String buildingSign_new = "VOBd" + sign_new + buildingSign_.substring(projectSign_.length());
                        building.setBuildingSign(buildingSign_new);
                    }
                }
            }

            if (ConstantCE.servicedata_use_survey) {
                String projectSign = survey.getProjectSign();
                String collectBuildingSign = projectSign.substring(2);
                Entity_ci_instance instance = SurveyUtil.Generate_servicedata(collectBuildingSign, survey.getMeterArray());
                ci_instance_Update(instance);
                if (ConstantWeb.name2interval.containsKey("servicedata_max")) {
                    Entity_ci_instance instance_max = SurveyUtil.Convert_max(instance);
                    ci_instance_Update(instance_max);
                }
            }

            List<Entity_ci_instance> instanceList = SurveyUtil.Generate_EMS(survey);
            for (Entity_ci_instance instance : instanceList) {
                if (ConstantWeb.EMS_use_collectsign) {
                    String object_sign = instance.getObject_sign();
                    if (object_sign.startsWith("Bd")) {
                        instance.setObject_sign(object_sign.substring("Bd".length()));
                    } else if (object_sign.startsWith("VOBd")) {
                        instance.setObject_sign(object_sign.substring("VOBd".length()));
                    }
                }
                ci_instance_Update(instance);
                if (ConstantWeb.name2interval.containsKey("EMS_max")) {
                    Entity_ci_instance instance_max = SurveyUtil.Convert_max(instance);
                    ci_instance_Update(instance_max);
                }
            }

            return result;
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            throw e;
        }
    }

    public static synchronized void Survey_Delete2(TSurvey survey) throws Exception {
        try {
            String instance_name = "EMS";
            String object_sign = survey.getProjectSign();
            ci_instance_Delete(object_sign, instance_name);
            if (ConstantWeb.name2interval.containsKey("EMS_max")) {
                ci_instance_Delete(object_sign, "EMS_max");
            }
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            throw e;
        }
    }

    public static synchronized TSurveyResult Survey_Update2(TSurvey survey) throws Exception {
        String time_period = "15min";
        String system = "EMS";
        try {
            TSurveyResult result = SurveyUtil2.Check(survey);

            Map<String, Long> defaultMap = new HashMap<String, Long>();
            for (TDefaultFunctionId defa : survey.getDefaultFunctionIdArray()) {
                defaultMap.put(defa.getEnergyType(), defa.getFuncid());
            }
            Map<String, Boolean> reportMeterMap = new HashMap<String, Boolean>();
            for (TMeter meter : survey.getReportMeterArray()) {
                reportMeterMap.put(meter.getSign() + "_" + meter.getFuncid(), true);
            }
            Map<String, Boolean> InputMap = new HashMap<String, Boolean>();
            Map<String, Double> InputCTMap = new HashMap<String, Double>();
            SurveyUtil2.refresh(survey, defaultMap, InputMap, InputCTMap);
            List<WSEntity_point> pointList = new ArrayList<WSEntity_point>();
            delete_dy_pointlist(survey.getProjectSign(), system, false);
            delete_dy_pointlist(survey.getProjectSign(), system, true);
            // 采集点位
            {
                List<WSEntity_point> dy_pointList = new ArrayList<WSEntity_point>();
                Map<String, Boolean> ExistPointMap = new HashMap<String, Boolean>();
                if (survey.getPointArray() != null) {
                    for (TMeter pm : survey.getPointArray()) {
                        WSEntity_point point = new WSEntity_point();
                        point.setMeter(pm.getSign());
                        point.setFuncid(pm.getFuncid());
                        point.setData_type(pm.getDataType());
                        point.setCollect_cycle(pm.getTimeType());
                        point.setSystem(system);
                        point.setSource(pm.getSource());
                        point.setAddress_1(pm.getAddress_1());
                        point.setIp_port(pm.getIp_port());
                        point.setAddress_2(pm.getAddress_2());
                        point.setAddress_3(pm.getAddress_3());
                        point.setAddress_4(pm.getAddress_4());
                        point.setUnit(pm.getUnit());
                        point.setRemark_1(pm.getRemark_1());
                        point.setRemark_2(pm.getRemark_2());
                        point.setRemark_3(pm.getRemark_3());
                        dy_pointList.add(point);
                        ExistPointMap.put(pm.getSign() + "_" + pm.getFuncid(), true);
                    }
                }
                for (String input_sign : InputMap.keySet()) {
                    if (ExistPointMap.containsKey(input_sign)) {
                        continue;
                    }
                    if (reportMeterMap.containsKey(input_sign)) {
                        continue;
                    }
                    int index_ = input_sign.lastIndexOf('_');
                    String meter = input_sign.substring(0, index_);
                    long funcid = Long.parseLong(input_sign.substring(index_ + 1));
                    WSEntity_point point = new WSEntity_point();
                    point.setMeter(meter);
                    point.setFuncid(funcid);
                    point.setData_type("Acc");
                    point.setCollect_cycle(time_period);
                    point.setSystem(system);
                    dy_pointList.add(point);
                }
                if (dy_pointList.size() > 0) {
                    WSEntity_points points = new WSEntity_points();
                    points.setBuilding(survey.getProjectSign());
                    points.setPoints(dy_pointList.toArray(new WSEntity_point[0]));
                    set_dy_pointlist(points, false, system);
                }
                pointList.addAll(dy_pointList);
            }
            // 填报点位
            {
                List<WSEntity_point> report_pointList = new ArrayList<WSEntity_point>();
                for (TMeter meter : survey.getReportMeterArray()) {
                    WSEntity_point point = new WSEntity_point();
                    point.setMeter(meter.getSign());
                    point.setFuncid(meter.getFuncid());
                    point.setData_type(meter.getDataType());
                    if (ConstantWeb.reportpoint_fixed_period_enable) {
                        point.setCollect_cycle(ConstantWeb.reportpoint_fixed_period_value);
                    } else {
                        point.setCollect_cycle(meter.getTimeType());
                    }
                    point.setSystem(system);
                    report_pointList.add(point);
                }
                JSONObject InsertObject = new JSONObject();
                InsertObject.put("building", survey.getProjectSign());
                InsertObject.put("round_exist", 0L);
                InsertObject.put("round_used", 0L);
                QueryUtil.insert(ConstantCE.database_name, "originalreport_round", InsertObject);
                if (report_pointList.size() > 0) {
                    WSEntity_points points = new WSEntity_points();
                    points.setBuilding(survey.getProjectSign());
                    points.setPoints(report_pointList.toArray(new WSEntity_point[0]));
                    set_dy_pointlist(points, true, system);
                }
                pointList.addAll(report_pointList);
            }
            // servicedata
            {
                String projectSign = survey.getProjectSign();
                Entity_ci_instance instance = SurveyUtil2.Generate_servicedata(projectSign, pointList, InputCTMap);
                ci_instance_Update(instance);
                if (ConstantWeb.name2interval.containsKey("servicedata_max")) {
                    Entity_ci_instance instance_max = SurveyUtil.Convert_max(instance);
                    ci_instance_Update(instance_max);
                }
            }
            // EMS
            {
                Entity_ci_instance instance = SurveyUtil2.Generate_EMS(survey, defaultMap, InputMap);
                ci_instance_Update(instance);
                if (ConstantWeb.name2interval.containsKey("EMS_max")) {
                    Entity_ci_instance instance_max = SurveyUtil.Convert_max(instance);
                    ci_instance_Update(instance_max);
                }
            }

            return result;
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            throw e;
        }
    }

    public static synchronized WSEntity_ci_status ci_instance_status(String object_sign, String instance_name) throws Exception {
        JSONObject Criteria = new JSONObject();
        Criteria.put("object_sign", object_sign);
        Criteria.put("instance_name", instance_name);
        JSONArray Content = QueryUtil.select(ConstantCE.database_name, "ci_status", Criteria);
        if (Content.size() > 0) {
            JSONObject json = (JSONObject) Content.get(0);
            String status_request = (String) json.get("status_request");
            String status_current = null;
            if (json.containsKey("status_current")) {
                status_current = (String) json.get("status_current");
            }
            WSEntity_ci_status result = new WSEntity_ci_status();
            result.setObject_sign(object_sign);
            result.setInstance_name(instance_name);
            result.setStatus_request(status_request);
            result.setStatus_current(status_current);
            return result;
        } else {
            return null;
        }
    }

    public static synchronized WSEntity_points get_dy_pointlist(String building, boolean is_report) throws Exception {
        String dy_table = is_report ? "dy_reportpointlist" : "dy_pointlist";

        WSEntity_points result = new WSEntity_points();
        result.setBuilding(building);
        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        JSONArray content = QueryUtil.select(ConstantCE.database_name, dy_table, Criteria);
        List<WSEntity_point> pointList = new ArrayList<WSEntity_point>();
        for (int i = 0; i < content.size(); i++) {
            JSONObject item = (JSONObject) content.get(i);
            WSEntity_point point = new WSEntity_point();
            point.setMeter((String) item.get("meter"));
            point.setFuncid((Long) item.get("funcid"));
            point.setData_type((String) item.get("data_type"));
            point.setCollect_cycle((String) item.get("collect_cycle"));
            pointList.add(point);
        }
        result.setPoints(pointList.toArray(new WSEntity_point[0]));
        return result;
    }

    public static synchronized String get_fjd_0_buildingcomputetime(String building) throws Exception {
        String result = null;
        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        JSONArray content = QueryUtil.select(ConstantCE.database_name, "fjd_0_buildingcomputetime", Criteria);
        if (content.size() == 1) {
            JSONObject json = (JSONObject) content.get(0);
            result = (String) json.get("compute_time");
        }
        return result;
    }

    public static synchronized void fjd_0_buildingcomputetime_delete(String building) throws Exception {
        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        QueryUtil.delete(ConstantCE.database_name, "fjd_0_buildingcomputetime", Criteria);
    }

    public static void fjd_0_compute_request(String building, String compute_time_from, String compute_time_to) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        QueryUtil.delete(ConstantCE.database_name, "fjd_0_compute_request", Criteria);
        {
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("building", building);
            InsertObject.put("compute_time_from", compute_time_from);
            InsertObject.put("compute_time_to", compute_time_to);
            InsertObject.put("is_processed", 0L);
            InsertObject.put("operate_time", sdf.format(new Date()));
            QueryUtil.insert(ConstantCE.database_name, "fjd_0_compute_request", InsertObject);
        }
    }

    public static void fjd_0_compute_request_by_model(String building, String modelSign, String compute_time_from, String compute_time_to)
            throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

        String object_sign = building + "-001";
        String instance_name = "EMS";
        Entity_ci_instance Entity_ci_instance = InstanceUtil.Get(ConstantCE.database_name, object_sign, instance_name);
        CoreInstance instance = CoreInstance.Convert(Entity_ci_instance);
        CoreComputeUtil.Refresh(instance);
        CoreComputeUtilUtil.BuildNetwork(instance);
        List<CoreOutput> outputList = instance.outputList;
        Map<String, Boolean> all_input = new HashMap<String, Boolean>();
        for (CoreOutput output : outputList) {
            if (output.output_sign.startsWith("energydata_" + building + "_" + modelSign + "_")) {
                Iterator<String> keyIter = output.all_input.keySet().iterator();
                while (keyIter.hasNext()) {
                    String key = keyIter.next();
                    all_input.put(key, true);
                }
            }
        }
        String[] inputs = all_input.keySet().toArray(new String[0]);
        Arrays.sort(inputs);
        JSONArray points = new JSONArray();
        for (int i = 0; i < inputs.length; i++) {
            String input = inputs[i];
            if (!input.startsWith("servicedata_")) {
                continue;
            }
            if (input.startsWith("servicedata_virtualpoint_")) {
                continue;
            }
            points.add(input.substring("servicedata_".length()));
        }
        JSONObject detail_content = new JSONObject();
        detail_content.put("ContentType", "points");
        detail_content.put("points", points);

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        QueryUtil.delete(ConstantCE.database_name, "fjd_0_compute_request", Criteria);
        {
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("building", building);
            InsertObject.put("compute_time_from", compute_time_from);
            InsertObject.put("compute_time_to", compute_time_to);
            InsertObject.put("detail_content", detail_content.toJSONString());
            InsertObject.put("is_processed", 0L);
            InsertObject.put("operate_time", sdf.format(new Date()));
            QueryUtil.insert(ConstantCE.database_name, "fjd_0_compute_request", InsertObject);
        }
    }

    public static synchronized String servicedata_recompute(String object_sign, String time) throws Exception {
        JSONObject Criteria = new JSONObject();
        Criteria.put("object_sign", object_sign);
        Criteria.put("instance_name", "servicedata");
        JSONArray status = QueryUtil.select(ConstantCE.database_name, "ci_status", Criteria);
        if (status.size() != 1) {
            return "ci_status not exist";
        }
        JSONObject json = (JSONObject) status.get(0);
        String status_request = (String) json.get("status_request");
        if (!status_request.equals("stop")) {
            return "status_request must be stop, but it is " + status_request;
        }
        String status_current = null;
        if (json.containsKey("status_current")) {
            status_current = (String) json.get("status_current");
        }
        if (status_current != null && status_current.contains("running")) {
            return "status_current must not be running";
        }

        QueryUtil.delete(ConstantCE.database_name, "ce_preroundindex", Criteria);
        Criteria = new JSONObject();
        Criteria.put("building", object_sign);
        JSONArray fjd_0_computelog_array = QueryUtil.select(ConstantCE.database_name, "fjd_0_computelog", Criteria);
        if (fjd_0_computelog_array.size() > 0) {
            for (int i = fjd_0_computelog_array.size() - 1; i >= 0; i--) {
                JSONObject last = (JSONObject) fjd_0_computelog_array.get(i);
                long compute_round = (Long) last.get("compute_round");
                String compute_time_from = (String) last.get("compute_time_from");
                if (compute_time_from.compareTo(time) <= 0 && compute_round > 1) {
                    JSONObject InsertObject = new JSONObject();
                    InsertObject.put("object_sign", object_sign);
                    InsertObject.put("instance_name", "servicedata");
                    InsertObject.put("source_object_sign", object_sign);
                    InsertObject.put("source_instance_name", "fjd_0");
                    InsertObject.put("compute_round", compute_round - 1);
                    QueryUtil.insert(ConstantCE.database_name, "ce_preroundindex", InsertObject);
                    break;
                }
            }
        }
        JSONArray fjd_1_computelog_array = QueryUtil.select(ConstantCE.database_name, "fjd_1_computelog", Criteria);
        if (fjd_1_computelog_array.size() > 0) {
            JSONObject last = (JSONObject) fjd_1_computelog_array.get(fjd_1_computelog_array.size() - 1);
            long compute_round = (Long) last.get("compute_round");
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("object_sign", object_sign);
            InsertObject.put("instance_name", "servicedata");
            InsertObject.put("source_object_sign", object_sign);
            InsertObject.put("source_instance_name", "fjd_1");
            InsertObject.put("compute_round", compute_round);
            QueryUtil.insert(ConstantCE.database_name, "ce_preroundindex", InsertObject);
        }
        JSONArray fjd_2_computelog_array = QueryUtil.select(ConstantCE.database_name, "fjd_2_computelog", Criteria);
        if (fjd_2_computelog_array.size() > 0) {
            JSONObject last = (JSONObject) fjd_2_computelog_array.get(fjd_2_computelog_array.size() - 1);
            long compute_round = (Long) last.get("compute_round");
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("object_sign", object_sign);
            InsertObject.put("instance_name", "servicedata");
            InsertObject.put("source_object_sign", object_sign);
            InsertObject.put("source_instance_name", "fjd_2");
            InsertObject.put("compute_round", compute_round);
            QueryUtil.insert(ConstantCE.database_name, "ce_preroundindex", InsertObject);
        }

        return "success";
    }

    public static synchronized String EMS_recompute(String object_sign, String time) throws Exception {
        JSONObject Criteria = new JSONObject();
        Criteria.put("object_sign", object_sign);
        Criteria.put("instance_name", "EMS");
        JSONArray status = QueryUtil.select(ConstantCE.database_name, "ci_status", Criteria);
        if (status.size() != 1) {
            return "ci_status not exist";
        }
        JSONObject json = (JSONObject) status.get(0);
        String status_request = (String) json.get("status_request");
        if (!status_request.equals("stop")) {
            return "status_request must be stop, but it is " + status_request;
        }
        String status_current = null;
        if (json.containsKey("status_current")) {
            status_current = (String) json.get("status_current");
        }
        if (status_current != null && status_current.contains("running")) {
            return "status_current must not be running";
        }

        QueryUtil.delete(ConstantCE.database_name, "ce_preroundindex", Criteria);
        String building = object_sign.substring(0, object_sign.length() - 4);
        Criteria = new JSONObject();
        Criteria.put("object_sign", building);
        Criteria.put("instance_name", "servicedata");
        JSONArray fjd_0_computelog_array = QueryUtil.select(ConstantCE.database_name, "ce_computelog", Criteria);
        if (fjd_0_computelog_array.size() > 0) {
            for (int i = fjd_0_computelog_array.size() - 1; i >= 0; i--) {
                JSONObject last = (JSONObject) fjd_0_computelog_array.get(i);
                long compute_round = (Long) last.get("compute_round");
                String compute_time_from = (String) last.get("compute_time_from");
                if (compute_time_from.compareTo(time) <= 0 && compute_round > 1) {
                    JSONObject CriteriaInner = new JSONObject();
                    CriteriaInner.put("object_sign", building);
                    CriteriaInner.put("instance_name", "servicedata");
                    CriteriaInner.put("compute_round", compute_round);
                    JSONArray ContentInner = QueryUtil.select(ConstantCE.database_name, "ce_computeinputlog",
                            CriteriaInner);
                    boolean has_fjd_0 = false;
                    for (int ii = 0; ii < ContentInner.size(); ii++) {
                        JSONObject itemInner = (JSONObject) ContentInner.get(ii);
                        String source_instance_name = (String) itemInner.get("source_instance_name");
                        if (source_instance_name.equals("fjd_0")) {
                            has_fjd_0 = true;
                            break;
                        }
                    }

                    if (has_fjd_0) {
                        JSONObject InsertObject = new JSONObject();
                        InsertObject.put("object_sign", object_sign);
                        InsertObject.put("instance_name", "EMS");
                        InsertObject.put("source_object_sign", building);
                        InsertObject.put("source_instance_name", "servicedata");
                        InsertObject.put("compute_round", compute_round - 1);
                        QueryUtil.insert(ConstantCE.database_name, "ce_preroundindex", InsertObject);
                        break;
                    }
                }
            }
        }

        return "success";
    }

    private static void recompute_check_stop(String object_sign, String instance_name) throws Exception {
        JSONObject Criteria = new JSONObject();
        Criteria.put("object_sign", object_sign);
        Criteria.put("instance_name", instance_name);
        JSONArray status = QueryUtil.select(ConstantCE.database_name, "ci_status", Criteria);
        if (status.size() > 0) {
            JSONObject json = (JSONObject) status.get(0);
            String status_request = (String) json.get("status_request");
            if (!status_request.equals("stop")) {
                throw new Exception(object_sign + " " + instance_name + " " + "status_request must be stop, but it is " + status_request);
            }
        }
    }

    private static void recompute_reportpoint(String building, String compute_time, JSONArray dy_point_rel) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        Calendar calendar = Calendar.getInstance();
        JSONArray originalreport_round;
        {
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", building);
            originalreport_round = QueryUtil.select(ConstantCE.database_name, "originalreport_round", Criteria);
            if (originalreport_round.size() == 0) {
                return;
            }
        }
        JSONObject roundJSON = (JSONObject) originalreport_round.get(0);
        Long round_used = (Long) roundJSON.get("round_used");
        Map<String, ReportRecompute> itemMap = new HashMap<String, ReportRecompute>();
        for (int i = 0; i < dy_point_rel.size(); i++) {
            JSONObject dy_point = dy_point_rel.getJSONObject(i);
            String meter = (String) dy_point.get("meter");
            long funcid = (Long) dy_point.get("funcid");
            JSONObject CriteriaInner = new JSONObject();
            CriteriaInner.put("building", building);
            if (round_used != null) {
                JSONObject Criteria_round = new JSONObject();
                Criteria_round.put("$lte", round_used);
                CriteriaInner.put("round", Criteria_round);
            }
            JSONObject Criteria_receivetime = new JSONObject();
            Criteria_receivetime.put("$gt", compute_time);
            CriteriaInner.put("receivetime", Criteria_receivetime);
            CriteriaInner.put("meter", meter);
            CriteriaInner.put("funcid", funcid);
            JSONArray originalreport_log = QueryUtil.select(ConstantCE.database_name, "originalreport_log",
                    CriteriaInner);
            for (int ii = 0; ii < originalreport_log.size(); ii++) {
                JSONObject logInner = originalreport_log.getJSONObject(ii);
                long round = (Long) logInner.get("round");
                String receivetime = (String) logInner.get("receivetime");
                Date parseTime = sdf.parse(receivetime);
                calendar.setTime(parseTime);
                calendar.set(Calendar.MONTH, calendar.get(Calendar.MONTH) / 3 * 3);
                calendar.set(Calendar.DAY_OF_MONTH, 1);
                calendar.set(Calendar.HOUR_OF_DAY, 0);
                calendar.set(Calendar.MINUTE, 0);
                calendar.set(Calendar.SECOND, 0);
                Date time1 = calendar.getTime();
                String time1String = sdf.format(time1);
                if (!itemMap.containsKey(time1String)) {
                    itemMap.put(time1String, new ReportRecompute(time1));
                }
                ReportRecompute reportRecomputer = itemMap.get(time1String);
                if (reportRecomputer.time_from == null || reportRecomputer.time_from.after(parseTime)) {
                    reportRecomputer.time_from = parseTime;
                }
                if (reportRecomputer.time_to == null || reportRecomputer.time_to.before(parseTime)) {
                    reportRecomputer.time_to = parseTime;
                }
                reportRecomputer.roundList.add(round);
                if (!reportRecomputer.pointMap.containsKey(meter + "-" + funcid)) {
                    reportRecomputer.pointMap.put(meter + "-" + funcid, new ReportRecomputeItem(meter, funcid));
                }
                ReportRecomputeItem computeItem = reportRecomputer.pointMap.get(meter + "-" + funcid);
                if (computeItem.time_from == null || computeItem.time_from.after(parseTime)) {
                    computeItem.time_from = parseTime;
                }
                if (computeItem.time_to == null || computeItem.time_to.before(parseTime)) {
                    computeItem.time_to = parseTime;
                }
                computeItem.roundList.add(round);
            }
        }
        String[] keys = itemMap.keySet().toArray(new String[0]);
        Arrays.sort(keys);
        String object_sign = building;
        String instance_name = "servicedata";
        Long curround = 0L;
        {
            JSONObject Criteria = new JSONObject();
            Criteria.put("object_sign", object_sign);
            Criteria.put("instance_name", instance_name);
            JSONArray resultContent = QueryUtil.select(ConstantCE.database_name, "ce_curround", Criteria);
            if (resultContent.size() > 0) {
                JSONObject ce_curround = (JSONObject) resultContent.get(0);
                curround = (Long) ce_curround.get("compute_round");
            }
        }
        for (String key : keys) {
            curround++;

            ReportRecompute item = itemMap.get(key);
            Generate_round(ConstantCE.database_name, building, instance_name, curround);

            JSONObject log = new JSONObject();
            log.put("object_sign", object_sign);
            log.put("instance_name", instance_name);
            log.put("compute_round", curround);
            log.put("compute_time_from", sdf.format(item.time_from));
            log.put("compute_time_to", sdf.format(item.time_to));
            log.put("is_finished", 0L);
            QueryUtil.insert(ConstantCE.database_name, "ce_computelog", log);

            JSONObject inutlog = new JSONObject();
            inutlog.put("object_sign", object_sign);
            inutlog.put("instance_name", instance_name);
            inutlog.put("compute_round", curround);
            inutlog.put("source_object_sign", building);
            inutlog.put("source_instance_name", "report");
            StringBuffer sb = new StringBuffer();
            for (Long roundInner : item.roundList) {
                if (sb.length() > 0) {
                    sb.append(',');
                }
                sb.append(roundInner);
            }
            inutlog.put("compute_rounds", sb.toString());
            QueryUtil.insert(ConstantCE.database_name, "ce_computeinputlog", inutlog);
            for (String point : item.pointMap.keySet()) {
                ReportRecomputeItem meter_funcid = item.pointMap.get(point);
                JSONObject inputdetail = new JSONObject();
                inputdetail.put("object_sign", object_sign);
                inputdetail.put("instance_name", instance_name);
                inputdetail.put("compute_round", curround);
                inputdetail.put("input_sign", "meterdata_" + meter_funcid.meter + "_" + meter_funcid.funcid);
                inputdetail.put("valid_time_from", sdf.format(meter_funcid.time_from));
                inputdetail.put("valid_time_to", sdf.format(meter_funcid.time_to));
                QueryUtil.insert(ConstantCE.database_name, "ce_computeinputdetail", inputdetail);
            }
        }
    }

    private static void Generate_round(String database_name, String object_sign, String instance_name, Long round_new)
            throws Exception {
        JSONObject Criteria = new JSONObject();
        Criteria.put("object_sign", object_sign);
        Criteria.put("instance_name", instance_name);
        JSONArray resultContent = QueryUtil.select(database_name, "ce_curround", Criteria);
        long compute_round;
        if (round_new == null) {
            long exist = 0;
            if (resultContent.size() > 0) {
                JSONObject roundmax = (JSONObject) resultContent.get(0);
                exist = (Long) roundmax.get("compute_round");
            }
            compute_round = exist + 1;
        } else {
            compute_round = round_new;
        }

        if (resultContent.size() > 0) {
            JSONObject Set = new JSONObject();
            Set.put("compute_round", compute_round);
            QueryUtil.update(database_name, "ce_curround", Criteria, Set);
        } else {
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("object_sign", object_sign);
            InsertObject.put("instance_name", instance_name);
            InsertObject.put("compute_round", compute_round);
            QueryUtil.insert(database_name, "ce_curround", InsertObject);
        }
    }

    public static synchronized void set_fjd_0_buildingcomputetime(String building, String compute_time, boolean initialize) throws Exception {
        recompute_check_stop(building, "fjd_0_Acc_Inst_EnumV");

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);

        JSONArray dy_point_rel = QueryUtil.select(ConstantCE.database_name, "dy_reportpointlist", Criteria);
        if (dy_point_rel.size() > 0) {
            recompute_check_stop(building, "servicedata");
            recompute_reportpoint(building, compute_time, dy_point_rel);
        }

        {
            JSONArray content = QueryUtil.select(ConstantCE.database_name, "fjd_0_buildingcomputetime", Criteria);
            if (content.size() == 1) {
                JSONObject Set = new JSONObject();
                Set.put("compute_time", compute_time);
                if (initialize) {
                    Set.put("first_compute_time", compute_time);
                }
                QueryUtil.update(ConstantCE.database_name, "fjd_0_buildingcomputetime", Criteria, Set);
            } else {
                JSONObject InsertObject = new JSONObject();
                InsertObject.put("building", building);
                InsertObject.put("compute_time", compute_time);
                InsertObject.put("first_compute_time", compute_time);
                QueryUtil.insert(ConstantCE.database_name, "fjd_0_buildingcomputetime", InsertObject);
            }
        }
        {
            JSONArray content = QueryUtil.select(ConstantCE.database_name, "fjd_0_metercomputetime", Criteria);
            for (int i = 0; i < content.size(); i++) {
                JSONObject item = (JSONObject) content.get(i);
                String meter = (String) item.get("meter");
                long funcid = (Long) item.get("funcid");
                if (!ConstantWeb.recompute_metercomputetime) {
                    String next_data_time = (String) item.get("next_data_time");
                    if (next_data_time.compareTo(compute_time) <= 0) {
                        continue;
                    }
                }
                JSONObject Set = new JSONObject();
                Set.put("next_data_time", compute_time);
                JSONObject CriteriaInner = new JSONObject();
                CriteriaInner.put("building", building);
                CriteriaInner.put("meter", meter);
                CriteriaInner.put("funcid", funcid);
                QueryUtil.update(ConstantCE.database_name, "fjd_0_metercomputetime", CriteriaInner, Set);
            }
        }
    }

    public static synchronized String set_fjd_0_buildingcomputetime_by_model(String building, String energyModelSign, String compute_time)
            throws Exception {
        recompute_check_stop(building, "fjd_0_Acc_Inst_EnumV");

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);

        JSONArray dy_reportpointlist = QueryUtil.select(ConstantCE.database_name, "dy_reportpointlist", Criteria);
        JSONArray dy_point_rel = new JSONArray();

        String object_sign = building + "-001";
        String instance_name = "EMS";
        Entity_ci_instance Entity_ci_instance = InstanceUtil.Get(ConstantCE.database_name, object_sign, instance_name);
        CoreInstance instance = CoreInstance.Convert(Entity_ci_instance);
        CoreComputeUtil.Refresh(instance);
        CoreComputeUtilUtil.BuildNetwork(instance);
        List<CoreOutput> outputList = instance.outputList;
        int energy_item_count = 0;
        Map<String, Boolean> all_input = new HashMap<String, Boolean>();
        for (CoreOutput output : outputList) {
            if (output.output_sign.startsWith("energydata_" + building + "_" + building + energyModelSign + "_")) {
                energy_item_count++;
                Iterator<String> keyIter = output.all_input.keySet().iterator();
                while (keyIter.hasNext()) {
                    String key = keyIter.next();
                    all_input.put(key, true);
                    for (int i = 0; i < dy_reportpointlist.size(); i++) {
                        JSONObject dy_point = dy_reportpointlist.getJSONObject(i);
                        String meter = (String) dy_point.get("meter");
                        long funcid = (Long) dy_point.get("funcid");
                        if (key.equals("servicedata_" + meter + "_" + funcid)) {
                            dy_point_rel.add(dy_point);
                        }
                    }
                }
            }
        }
        if (dy_point_rel.size() > 0) {
            recompute_check_stop(building, "servicedata");
            recompute_reportpoint(building, compute_time, dy_point_rel);
        }

        JSONArray RelatedPoints = new JSONArray();
        JSONArray UpdatePoints = new JSONArray();
        JSONArray Content = QueryUtil.select(ConstantCE.database_name, "fjd_0_metercomputetime", Criteria);
        for (int i = 0; i < Content.size(); i++) {
            JSONObject item = (JSONObject) Content.get(i);
            String meter = (String) item.get("meter");
            long funcid = (Long) item.get("funcid");
            JSONObject point = new JSONObject();
            point.put("meter", meter);
            point.put("funcid", funcid);
            String input_sign = "servicedata_" + meter + "_" + funcid;
            if (all_input.containsKey(input_sign)) {
                RelatedPoints.add(point);
                String next_data_time = (String) item.get("next_data_time");
                if (next_data_time.compareTo(compute_time) > 0) {
                    UpdatePoints.add(point);
                    JSONObject CriteriaInner = new JSONObject();
                    CriteriaInner.put("building", building);
                    CriteriaInner.put("meter", meter);
                    CriteriaInner.put("funcid", funcid);
                    JSONObject SetInner = new JSONObject();
                    SetInner.put("next_data_time", compute_time);
                    QueryUtil.update(ConstantCE.database_name, "fjd_0_metercomputetime", CriteriaInner, SetInner);
                }
            }
        }

        JSONObject result = new JSONObject();
        result.put("EnergyModelNodeCount", energy_item_count);
        result.put("RelatedPoints", RelatedPoints);
        result.put("UpdatePoints", UpdatePoints);
        return FastJsonUtil.toFormatString(result);
    }

    public static synchronized String set_dy_pointlist(WSEntity_points points, boolean is_report, String system) throws Exception {
        String dy_table = is_report ? "dy_reportpointlist" : "dy_pointlist";

        {
            String object_sign = points.getBuilding();
            JSONObject Criteria = new JSONObject();
            Criteria.put("object_sign", object_sign);
            Criteria.put("instance_name", "fjd_0_Acc_Inst_EnumV");
            JSONArray status = QueryUtil.select(ConstantCE.database_name, "ci_status", Criteria);
            if (status.size() > 0) {
                JSONObject json = (JSONObject) status.get(0);
                String status_request = (String) json.get("status_request");
                if (!status_request.equals("stop")) {
                    throw new Exception("status_request must be stop, but it is " + status_request);
                }
            }
        }

        String building = points.getBuilding();
        delete_dy_pointlist(building, system, is_report);
        JSONArray InsertObjects = new JSONArray();
        for (WSEntity_point point : points.getPoints()) {
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("building", building);
            InsertObject.put("meter", point.getMeter());
            InsertObject.put("funcid", point.getFuncid());
            InsertObject.put("data_type", point.getData_type());
            if (is_report && ConstantWeb.reportpoint_fixed_period_enable) {
                InsertObject.put("collect_cycle", ConstantWeb.reportpoint_fixed_period_value);
            } else {
                InsertObject.put("collect_cycle", point.getCollect_cycle());
            }
            InsertObject.put("system", system);
            InsertObject.put("source", point.getSource());
            InsertObject.put("address_1", point.getAddress_1());
            InsertObject.put("address_2", point.getAddress_2());
            InsertObject.put("address_3", point.getAddress_3());
            InsertObject.put("address_4", point.getAddress_4());
            InsertObjects.add(InsertObject);
        }
        QueryUtil.batch_insert(ConstantCE.database_name, dy_table, InsertObjects);

        return "success";
    }

    public static synchronized void delete_dy_pointlist(String building, String system, boolean is_report) throws Exception {
        String dy_table = is_report ? "dy_reportpointlist" : "dy_pointlist";

        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        JSONArray Content = QueryUtil.select(ConstantCE.database_name, dy_table, Criteria);
        for (int i = 0; i < Content.size(); i++) {
            JSONObject item = (JSONObject) Content.get(i);
            String systemInner = (String) item.get("system");
            if (systemInner != null && systemInner.equals(system)) {
                String meter = (String) item.get("meter");
                long funcid = (Long) item.get("funcid");
                JSONObject CriteriaInner = new JSONObject();
                CriteriaInner.put("building", building);
                CriteriaInner.put("meter", meter);
                CriteriaInner.put("funcid", funcid);
                QueryUtil.delete(ConstantCE.database_name, dy_table, CriteriaInner);
            }
        }
    }

    public static synchronized void report(WSEntity_report pointdata, boolean is_energy, Date curr_time) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String data_type = is_energy ? "energy" : "read_data";
        if (is_energy) {
            pointdata.setReceivetime(pointdata.getTime_to());
        }

        {
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", pointdata.getBuilding());
            Criteria.put("meter", pointdata.getMeter());
            Criteria.put("funcid", pointdata.getFuncid());
            JSONArray queryResult = QueryUtil.select(ConstantCE.database_name, "dy_reportpointlist", Criteria);
            if (queryResult.size() == 0) {
                throw new Exception("point not exist,can't insert!");
            }
        }
        {
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", pointdata.getBuilding());
            Criteria.put("data_type", data_type);
            Criteria.put("meter", pointdata.getMeter());
            Criteria.put("funcid", pointdata.getFuncid());
            Criteria.put("receivetime", pointdata.getReceivetime());
            JSONArray queryResult = QueryUtil.select(ConstantCE.database_name, "originalreport_data", Criteria);
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("building", pointdata.getBuilding());
            InsertObject.put("data_type", data_type);
            InsertObject.put("meter", pointdata.getMeter());
            InsertObject.put("funcid", pointdata.getFuncid());
            InsertObject.put("receivetime", pointdata.getReceivetime());
            InsertObject.put("time_from", pointdata.getTime_from());
            InsertObject.put("time_to", pointdata.getTime_to());
            InsertObject.put("data", pointdata.getData());
            // 写data
            if (pointdata.getOperation().equals("delete") || pointdata.getOperation().equals("update")) {
                // if (queryResult.size() == 0) {
                // throw new Exception("pointdata not exist");
                // }
                if (pointdata.getOperation().equals("delete")) {
                    QueryUtil.delete(ConstantCE.database_name, "originalreport_data", Criteria);
                } else if (pointdata.getOperation().equals("update")) {
                    // JSONObject Set = new JSONObject();
                    // Set.put("data", pointdata.getData());
                    // QueryUtil.update(ConstantCE.database_name, "originalreport_data", Criteria, Set);
                    QueryUtil.delete(ConstantCE.database_name, "originalreport_data", Criteria);
                    QueryUtil.insert(ConstantCE.database_name, "originalreport_data", InsertObject);
                }
            } else if (pointdata.getOperation().equals("insert")) {
                if (queryResult.size() > 0) {
                    throw new Exception("pointdata exist,can't insert!");
                }
                QueryUtil.insert(ConstantCE.database_name, "originalreport_data", InsertObject);
            }
        }
        // 读round
        long round_exist = 0L;
        {
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", pointdata.getBuilding());
            JSONArray queryResult = QueryUtil.select(ConstantCE.database_name, "originalreport_round", Criteria);

            if (queryResult.size() > 0) {
                JSONObject roundItem = (JSONObject) queryResult.get(0);
                round_exist = (Long) roundItem.get("round_exist");
            } else {
                JSONObject InsertObject = new JSONObject();
                InsertObject.put("building", pointdata.getBuilding());
                InsertObject.put("round_exist", 0L);
                InsertObject.put("round_used", 0L);
                QueryUtil.insert(ConstantCE.database_name, "originalreport_round", InsertObject);
            }
        }
        round_exist++;
        // 写log
        {
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("building", pointdata.getBuilding());
            InsertObject.put("round", round_exist);
            InsertObject.put("operation", pointdata.getOperation());
            InsertObject.put("meter", pointdata.getMeter());
            InsertObject.put("funcid", pointdata.getFuncid());
            InsertObject.put("receivetime", pointdata.getReceivetime());
            InsertObject.put("data_type", data_type);
            InsertObject.put("time_from", pointdata.getTime_from());
            InsertObject.put("time_to", pointdata.getTime_to());
            InsertObject.put("operate_time", sdf.format(curr_time));
            InsertObject.put("data", pointdata.getData());
            QueryUtil.insert(ConstantCE.database_name, "originalreport_log", InsertObject);
        }
        // 写round
        JSONObject Criteria = new JSONObject();
        Criteria.put("building", pointdata.getBuilding());
        JSONObject Set = new JSONObject();
        Set.put("round_exist", round_exist);
        QueryUtil.update(ConstantCE.database_name, "originalreport_round", Criteria, Set);
    }

    public static synchronized WSEntity_datas get_reportdatalist(String building, String data_type, String meter, Long funcid, String time_from,
                                                                 String time_to) throws Exception {
        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        Criteria.put("data_type", data_type);
        Criteria.put("meter", meter);
        Criteria.put("funcid", funcid);
        JSONObject Range = new JSONObject();
        Range.put("$gte", time_from);
        Range.put("$lte", time_to);
        Criteria.put("receivetime", Range);
        JSONArray queryResult = QueryUtil.select(ConstantCE.database_name, "originalreport_data", Criteria);
        WSEntity_data[] data_array = new WSEntity_data[queryResult.size()];
        for (int i = 0; i < queryResult.size(); i++) {
            JSONObject data = (JSONObject) queryResult.get(i);
            data_array[i] = new WSEntity_data();
            data_array[i].setData_type((String) data.get("data_type"));
            if (data_array[i].getData_type().equals("read_data")) {
                data_array[i].setReceivetime((String) data.get("receivetime"));
            } else {
                data_array[i].setTime_from((String) data.get("time_from"));
                data_array[i].setTime_to((String) data.get("time_to"));
            }
            data_array[i].setData((Double) data.get("data"));
        }
        WSEntity_datas result = new WSEntity_datas();
        result.setBuilding(building);
        result.setMeter(meter);
        result.setFuncid(funcid);
        result.setData_array(data_array);
        return result;
    }

    public static synchronized void report_change(WSEntity_report_change pointdata) throws Exception {
        {
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", pointdata.getBuilding());
            Criteria.put("meter", pointdata.getMeter());
            Criteria.put("funcid", pointdata.getFuncid());
            JSONArray queryResult = QueryUtil.select(ConstantCE.database_name, "dy_reportpointlist", Criteria);
            if (queryResult.size() == 0) {
                throw new Exception("point not exist,can't insert!");
            }
        }
        {
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", pointdata.getBuilding());
            Criteria.put("meter", pointdata.getMeter());
            Criteria.put("funcid", pointdata.getFuncid());
            Criteria.put("receivetime", pointdata.getReceivetime());
            JSONArray queryResult = QueryUtil.select(ConstantCE.database_name, "originalreport_change_meter", Criteria);
            // 写data
            if (pointdata.getOperation().equals("delete") || pointdata.getOperation().equals("update")) {
                if (queryResult.size() == 0) {
                    throw new Exception("pointdata not exist");
                }
                if (pointdata.getOperation().equals("delete")) {
                    QueryUtil.delete(ConstantCE.database_name, "originalreport_change_meter", Criteria);
                } else if (pointdata.getOperation().equals("update")) {
                    JSONObject Set = new JSONObject();
                    Set.put("data_old", pointdata.getData_old());
                    Set.put("data_new", pointdata.getData_new());
                    QueryUtil.update(ConstantCE.database_name, "originalreport_change_meter", Criteria, Set);
                }
            } else if (pointdata.getOperation().equals("insert")) {
                if (queryResult.size() > 0) {
                    throw new Exception("pointdata exist,can't insert!");
                }
                JSONObject InsertObject = new JSONObject();
                InsertObject.put("building", pointdata.getBuilding());
                InsertObject.put("meter", pointdata.getMeter());
                InsertObject.put("funcid", pointdata.getFuncid());
                InsertObject.put("receivetime", pointdata.getReceivetime());
                InsertObject.put("data_old", pointdata.getData_old());
                InsertObject.put("data_new", pointdata.getData_new());
                QueryUtil.insert(ConstantCE.database_name, "originalreport_change_meter", InsertObject);
            }
        }
        // 读round
        long round_exist = 0L;
        {
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", pointdata.getBuilding());
            JSONArray queryResult = QueryUtil.select(ConstantCE.database_name, "originalreport_round", Criteria);

            if (queryResult.size() > 0) {
                JSONObject roundItem = (JSONObject) queryResult.get(0);
                round_exist = (Long) roundItem.get("round_exist");
            }
        }
        round_exist++;
        // 写log
        {
            JSONObject InsertObject = new JSONObject();
            InsertObject.put("building", pointdata.getBuilding());
            InsertObject.put("round", round_exist);
            InsertObject.put("operation", "change_" + pointdata.getOperation());
            InsertObject.put("meter", pointdata.getMeter());
            InsertObject.put("funcid", pointdata.getFuncid());
            InsertObject.put("receivetime", pointdata.getReceivetime());
            QueryUtil.insert(ConstantCE.database_name, "originalreport_log", InsertObject);
        }
        // 写round
        JSONObject Criteria = new JSONObject();
        Criteria.put("building", pointdata.getBuilding());
        JSONObject Set = new JSONObject();
        Set.put("round_exist", round_exist);
        QueryUtil.update(ConstantCE.database_name, "originalreport_round", Criteria, Set);
    }

    public static synchronized WSEntity_changes get_reportdatalist_change(String building) throws Exception {
        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        JSONArray queryResult = QueryUtil.select(ConstantCE.database_name, "originalreport_change_meter", Criteria);
        WSEntity_change[] data_array = new WSEntity_change[queryResult.size()];
        for (int i = 0; i < queryResult.size(); i++) {
            JSONObject data = (JSONObject) queryResult.get(i);
            data_array[i] = new WSEntity_change();
            data_array[i].setMeter((String) data.get("meter"));
            data_array[i].setFuncid((Long) data.get("funcid"));
            data_array[i].setReceivetime((String) data.get("receivetime"));
            data_array[i].setData_old((Double) data.get("data_old"));
            data_array[i].setData_new((Double) data.get("data_new"));
        }
        WSEntity_changes result = new WSEntity_changes();
        result.setBuilding(building);
        result.setData_array(data_array);
        return result;

    }
}
