package com.persagy.service;

import com.alibaba.fastjson.JSONObject;
import com.pbsage.parser.emsiv.EMSLexer;
import com.pbsage.parser.emsiv.EMSParser;
import com.pbsage.parser.emsiv.EMSScanner;
import com.persagy.util.ConstantAgent;
import com.persagy.util.ConstantWeb;
import com.persagy.wsentity.*;
import com.sagacloud.util.compute_engine.entity.Entity_ci_instance;
import com.sagacloud.util.compute_engine.entity.Entity_ci_instanceinput;
import com.sagacloud.util.compute_engine.entity.Entity_ci_instanceoutput;
import com.sagacloud.util.compute_engine.util.ConstantCE;
import lombok.extern.slf4j.Slf4j;
import org.antlr.runtime.ANTLRInputStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.tree.CommonTree;
import org.antlr.runtime.tree.CommonTreeNodeStream;

import java.io.ByteArrayInputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
@Slf4j
public class SurveyUtil {
	public static SSurveyResult Check(SSurvey survey) {
		SSurveyResult result = new SSurveyResult();
		result.setProjectSign(survey.getProjectSign());

		return result;
	}

	public static Entity_ci_instance Convert_max(Entity_ci_instance instance) throws Exception {
		String object_sign = instance.getObject_sign();
		String source_instance_name = instance.getInstance_name();
		String instance_name = instance.getInstance_name() + "_max";
		Entity_ci_instance result = new Entity_ci_instance();
		result.setObject_sign(object_sign);
		result.setInstance_name(instance_name);
		result.setFlag_valid(1L);
		result.setCompute_interval_seconds(ConstantWeb.name2interval.get(instance_name));

		List<Entity_ci_instanceinput> inputList = new ArrayList<Entity_ci_instanceinput>();
		List<Entity_ci_instanceoutput> outputList = new ArrayList<Entity_ci_instanceoutput>();

		Entity_ci_instanceoutput[] outputs_tmp = instance.getOutputs();
		for (Entity_ci_instanceoutput output_tmp : outputs_tmp) {
			Entity_ci_instanceinput input = new Entity_ci_instanceinput();
			input.setObject_sign(object_sign);
			input.setInstance_name(instance_name);
			input.setInput_sign(output_tmp.getOutput_sign());
			input.setTime_period(output_tmp.getTime_period());
			input.setData_class(output_tmp.getData_class());
			input.setDao_type(output_tmp.getDao_type());
			input.setDao_key(output_tmp.getDao_key());
			input.setSource_object_sign(object_sign);
			input.setSource_instance_name(source_instance_name);
			input.setSource_output_sign(output_tmp.getOutput_sign());
			inputList.add(input);

			Entity_ci_instanceoutput output = new Entity_ci_instanceoutput();
			output.setObject_sign(object_sign);
			output.setInstance_name(instance_name);
			output.setOutput_sign(output_tmp.getOutput_sign());
			output.setTime_begin(output_tmp.getTime_begin());
			output.setTime_end(output_tmp.getTime_end());
			output.setData_class(output_tmp.getData_class());
			output.setDao_type(output_tmp.getDao_type() + "_max");
			output.setDao_key(output_tmp.getDao_key());
			output.setNeed_all_var_data(false);
			output.setTime_period_intialized(true);
			output.setTime_period(output_tmp.getTime_period());
			output.setMode("expression");
			output.setExpression(output_tmp.getOutput_sign());
			outputList.add(output);
		}

		result.setInputs(inputList.toArray(new Entity_ci_instanceinput[0]));
		result.setOutputs(outputList.toArray(new Entity_ci_instanceoutput[0]));

		return result;
	}

	public static List<Entity_ci_instance> Generate_EMS(SSurvey survey) throws Exception {
		List<Entity_ci_instance> result = new ArrayList<Entity_ci_instance>();

		String projectSign = survey.getProjectSign();
		String collectBuildingSign = projectSign.substring(2);
		SBuilding[] buildingArray = survey.getBuildingArray();
		for (SBuilding building : buildingArray) {
			Entity_ci_instance instance = Generate_EMS(projectSign, collectBuildingSign, survey.getMeterArray(), building);
			result.add(instance);
		}

		return result;
	}

	public static Entity_ci_instance Generate_servicedata(String collectBuildingSign, SMeter[] meterArray) throws Exception {
		String time_period = "15min";

		String object_sign = collectBuildingSign;
		String instance_name = "servicedata";
		Entity_ci_instance instance = new Entity_ci_instance();
		instance.setObject_sign(object_sign);
		instance.setInstance_name(instance_name);
		instance.setFlag_valid(1L);
		instance.setCompute_interval_seconds(ConstantWeb.name2interval.get(instance_name));

		String building = object_sign;
		List<Entity_ci_instanceinput> inputList = new ArrayList<Entity_ci_instanceinput>();
		List<Entity_ci_instanceoutput> outputList = new ArrayList<Entity_ci_instanceoutput>();
		for (SMeter meter : meterArray) {
			String detail_meter = meter.getMeterSign();
			long detail_funcid = meter.getFuncid();
			{
				Entity_ci_instanceinput input = new Entity_ci_instanceinput();
				input.setObject_sign(object_sign);
				input.setInstance_name(instance_name);
				input.setInput_sign("meterdata_" + detail_meter + "_" + detail_funcid);
				input.setTime_period(time_period);
				input.setData_class("near");
				input.setDao_type("fjd_near");
				JSONObject dao_key = new JSONObject();
				dao_key.put("building", building);
				dao_key.put("meter", detail_meter);
				dao_key.put("funcid", detail_funcid);
				input.setDao_key(dao_key.toJSONString());
				input.setSource_object_sign(building);
				input.setSource_instance_name("fjd");
				input.setSource_output_sign(detail_meter + "_" + detail_funcid);
				inputList.add(input);
			}
			{
				Entity_ci_instanceoutput output = new Entity_ci_instanceoutput();
				output.setObject_sign(object_sign);
				output.setInstance_name(instance_name);
				output.setOutput_sign("servicedata_" + detail_meter + "_" + detail_funcid);
				output.setTime_begin("19700101000000");
				output.setTime_end("99990101000000");
				output.setData_class("consumption");
				output.setDao_type("servicedata");
				JSONObject dao_key = new JSONObject();
				dao_key.put("building", building);
				dao_key.put("meter", detail_meter);
				dao_key.put("funcid", detail_funcid);
				output.setDao_key(dao_key.toJSONString());
				output.setNeed_all_var_data(true);
				output.setTime_period_intialized(true);
				output.setTime_period(time_period);
				output.setMode("expression");
				String expression;
				if (meter.getCt() != null) {
					expression = "(meterdata_" + detail_meter + "_" + detail_funcid + "'+1-" + "meterdata_" + detail_meter + "_" + detail_funcid
							+ ")*" + meter.getCt();
				} else {
					expression = "meterdata_" + detail_meter + "_" + detail_funcid + "'+1-" + "meterdata_" + detail_meter + "_" + detail_funcid;
				}
				output.setExpression(expression);
				outputList.add(output);
			}
		}

		instance.setInputs(inputList.toArray(new Entity_ci_instanceinput[0]));
		instance.setOutputs(outputList.toArray(new Entity_ci_instanceoutput[0]));

		return instance;
	}

	private static Entity_ci_instance Generate_EMS(String projectSign, String collectBuildingSign, SMeter[] meterArray, SBuilding building)
			throws Exception {
		String time_period = "15min";

		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
		SimpleDateFormat sdf_ = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		String object_sign = building.getBuildingSign();
		String instance_name = "EMS";
		Entity_ci_instance instance = new Entity_ci_instance();
		instance.setObject_sign(object_sign);
		instance.setInstance_name(instance_name);
		instance.setFlag_valid(1L);
		instance.setCompute_interval_seconds(ConstantWeb.name2interval.get(instance_name));

		Map<String, SMeter> meterMap = new HashMap<String, SMeter>();
		for (SMeter meter : meterArray) {
			meterMap.put(meter.getMeterSign(), meter);
		}

		Map<String, SEnergyModel> EnergyModelMap = new HashMap<String, SEnergyModel>();
		SEnergyModel[] EnergyModelArray = building.getEnergyModelArray();
		for (SEnergyModel EnergyModel : EnergyModelArray) {
			EnergyModelMap.put(EnergyModel.getEnergyModelSign(), EnergyModel);
		}

		Map<String, SBuildingConfig> BuildingConfigMap = new HashMap<String, SBuildingConfig>();
		SBuildingConfig[] BuildingConfigArray = building.getBuildingConfigArray();
		for (SBuildingConfig energyModel : BuildingConfigArray) {
			BuildingConfigMap.put(energyModel.getBuildingConfigSign(), energyModel);
		}

		List<Entity_ci_instanceinput> inputList = new ArrayList<Entity_ci_instanceinput>();
		List<Entity_ci_instanceoutput> outputList = new ArrayList<Entity_ci_instanceoutput>();
		Map<String, Boolean> InputMap = new HashMap<String, Boolean>();
		SBuildingConfigUse[] buildingConfigUseArray = building.getBuildingConfigUseArray();
		for (SBuildingConfigUse bcu : buildingConfigUseArray) {
			SBuildingConfig BuildingConfig = BuildingConfigMap.get(bcu.getBuildingConfigSign());
			SEnergyModel EnergyModel = EnergyModelMap.get(BuildingConfig.getEnergyModelSign());
			SResearchNode[] ResearchNodeArray = BuildingConfig.getResearchNodeArray();
			Map<String, List<SResearchNode>> childrenMap = new HashMap<String, List<SResearchNode>>();
			for (SResearchNode node : ResearchNodeArray) {
				if (node.getParentSign() != null) {
					if (!childrenMap.containsKey(node.getParentSign())) {
						childrenMap.put(node.getParentSign(), new ArrayList<SResearchNode>());
					}
					childrenMap.get(node.getParentSign()).add(node);
				}
			}

			Map<String, String> unit2ExpressionMap = new HashMap<String, String>();
			SComputeUnit[] ComputeUnitArray = BuildingConfig.getComputeUnitArray();
			if (ComputeUnitArray != null) {
				for (SComputeUnit entity : ComputeUnitArray) {
					try {
						CommonTreeNodeStream nodes = GenerateParser(entity.getExpression());
						EMSScanner scanner = new EMSScanner(nodes);
						scanner.prog();
						String[] vars = scanner.GetVarDict().keySet().toArray(new String[0]);
						Map<String, String> old2newMap = new HashMap<String, String>();
						String new_expression = entity.getExpression();
						for (int index_var = 0; index_var < vars.length; index_var++) {
							String var = vars[index_var];
							String MeterSign;
							long FunctionTypeId;
							String varPart = var;
							int index_dot = varPart.indexOf('.');
							if (index_dot == -1) {
								MeterSign = varPart;
								FunctionTypeId = meterMap.get(MeterSign).getFuncid();
							} else {
								MeterSign = varPart.substring(0, index_dot);
								FunctionTypeId = Integer.parseInt(varPart.substring(index_dot + 1));
							}
							if (ConstantWeb.funcid_10101_to_0) {
								if (FunctionTypeId == 10101) {
									FunctionTypeId = 0;
								}
							}
							if (!InputMap.containsKey(MeterSign + "_" + FunctionTypeId)) {
								InputMap.put(MeterSign + "_" + FunctionTypeId, true);
							}
							if (!ConstantCE.servicedata_use_survey && meterMap.containsKey(MeterSign) && meterMap.get(MeterSign).getCt() != null) {
								old2newMap.put(var, "servicedata_" + MeterSign + "_" + FunctionTypeId + "*" + meterMap.get(MeterSign).getCt());
							} else {
								old2newMap.put(var, "servicedata_" + MeterSign + "_" + FunctionTypeId);
							}
						}
						for (int index_var = 0; index_var < vars.length; index_var++) {
							String var = vars[index_var];
							int index_dot = var.indexOf('.');
							if (index_dot == -1) {
								new_expression = new_expression.replaceAll("\\[" + var + "\\]", old2newMap.get(var));
							} else {
								new_expression = new_expression.replaceAll(
										"\\[" + var.substring(0, index_dot) + "\\." + var.substring(index_dot + 1) + "\\]", old2newMap.get(var));
							}
						}

						unit2ExpressionMap.put(entity.getSign(), new_expression);
					} catch (Exception e) {
						log.error(e.getMessage(),e);
					}
				}
			}

			Map<String, String> node2ExpressionMap = new HashMap<String, String>();
			SEnergyModelExpression[] EnergyModelExpressionArray = BuildingConfig.getEnergyModelExpressionArray();
			if (EnergyModelExpressionArray != null) {
				for (SEnergyModelExpression entity : EnergyModelExpressionArray) {
					try {
						CommonTreeNodeStream nodes = GenerateParser(entity.getExpression());
						EMSScanner scanner = new EMSScanner(nodes);
						scanner.prog();
						String[] vars = scanner.GetVarDict().keySet().toArray(new String[0]);
						Map<String, String> old2newMap = new HashMap<String, String>();
						String new_expression = entity.getExpression();
						for (int index_var = 0; index_var < vars.length; index_var++) {
							String var = vars[index_var];
							String MeterSign;
							long FunctionTypeId;
							String varPart = var;
							int index_dot = varPart.indexOf('.');
							if (index_dot == -1) {
								MeterSign = varPart;
								FunctionTypeId = meterMap.get(MeterSign).getFuncid();
							} else {
								MeterSign = varPart.substring(0, index_dot);
								FunctionTypeId = Integer.parseInt(varPart.substring(index_dot + 1));
							}
							if (ConstantWeb.funcid_10101_to_0) {
								if (FunctionTypeId == 10101) {
									FunctionTypeId = 0;
								}
							}
							if (!InputMap.containsKey(MeterSign + "_" + FunctionTypeId)) {
								InputMap.put(MeterSign + "_" + FunctionTypeId, true);
							}
							if (!ConstantCE.servicedata_use_survey && meterMap.containsKey(MeterSign) && meterMap.get(MeterSign).getCt() != null) {
								old2newMap.put(var, "servicedata_" + MeterSign + "_" + FunctionTypeId + "*" + meterMap.get(MeterSign).getCt());
							} else {
								old2newMap.put(var, "servicedata_" + MeterSign + "_" + FunctionTypeId);
							}
						}
						for (int index_var = 0; index_var < vars.length; index_var++) {
							String var = vars[index_var];
							int index_dot = var.indexOf('.');
							if (index_dot == -1) {
								new_expression = new_expression.replaceAll("\\[" + var + "\\]", old2newMap.get(var));
							} else {
								new_expression = new_expression.replaceAll(
										"\\[" + var.substring(0, index_dot) + "\\." + var.substring(index_dot + 1) + "\\]", old2newMap.get(var));
							}
						}
						node2ExpressionMap.put(entity.getSign(), new_expression);
					} catch (Exception e) {
						log.error(e.getMessage(),e);
					}
				}
			}

			Map<String, List<SResearchNode>> ModelChildrenMap = new HashMap<String, List<SResearchNode>>();
			for (SResearchNode node : ResearchNodeArray) {
				if (!childrenMap.containsKey(node.getSign())) {
					if (node.getEnergyModelNodeSign() != null) {
						if (!ModelChildrenMap.containsKey(node.getEnergyModelNodeSign())) {
							ModelChildrenMap.put(node.getEnergyModelNodeSign(), new ArrayList<SResearchNode>());
						}
						ModelChildrenMap.get(node.getEnergyModelNodeSign()).add(node);
					}
				}
			}

			Map<String, List<SEnergyModelNode>> Model2ModelListMap = new HashMap<String, List<SEnergyModelNode>>();
			for (SEnergyModelNode node : EnergyModel.getNodeArray()) {
				if (node.getParentSign() != null) {
					if (!Model2ModelListMap.containsKey(node.getParentSign())) {
						Model2ModelListMap.put(node.getParentSign(), new ArrayList<SEnergyModelNode>());
					}
					Model2ModelListMap.get(node.getParentSign()).add(node);
				}
			}

			Map<String, String> finishMap = new HashMap<String, String>();
			while (true) {
				boolean has_change = false;
				for (SEnergyModelNode node : EnergyModel.getNodeArray()) {
					if (finishMap.containsKey(node.getSign())) {
						continue;
					}

					boolean children_finsh = true;
					if (Model2ModelListMap.containsKey(node.getSign())) {
						List<SEnergyModelNode> children = Model2ModelListMap.get(node.getSign());
						for (SEnergyModelNode child : children) {
							if (!finishMap.containsKey(child.getSign())) {
								children_finsh = false;
								break;
							}
						}
					}

					if (children_finsh) {
						has_change = true;
						if (node2ExpressionMap.containsKey(node.getSign())) {
							finishMap.put(node.getSign(), node2ExpressionMap.get(node.getSign()));
						} else {
							StringBuffer sb = new StringBuffer();
							if (Model2ModelListMap.containsKey(node.getSign())) {
								List<SEnergyModelNode> children = Model2ModelListMap.get(node.getSign());
								for (SEnergyModelNode child : children) {
									String expression = finishMap.get(child.getSign());
									if (expression != null && expression.length() > 0) {
										if (sb.length() > 0) {
											sb.append("+");
										}
										sb.append("(" + finishMap.get(child.getSign()) + ")");
									}
								}
							}
							if (ModelChildrenMap.containsKey(node.getSign())) {
								List<SResearchNode> ResearchNodeList = ModelChildrenMap.get(node.getSign());
								for (SResearchNode ResearchNode : ResearchNodeList) {
									if (ResearchNode.getIsComputeUnit().equals("Y")) {
										if (sb.length() > 0) {
											sb.append("+");
										}
										sb.append("(" + unit2ExpressionMap.get(ResearchNode.getSign()) + ")");
									} else {
										if (ResearchNode.getMeter() != null) {
											String MeterSign = ResearchNode.getMeter();
											long FunctionTypeId = meterMap.get(MeterSign).getFuncid();
											if (ConstantWeb.funcid_10101_to_0) {
												if (FunctionTypeId == 10101) {
													FunctionTypeId = 0;
												}
											}
											if (!InputMap.containsKey(MeterSign + "_" + FunctionTypeId)) {
												InputMap.put(MeterSign + "_" + FunctionTypeId, true);
											}

											if (sb.length() > 0) {
												sb.append("+");
											}
											if (!ConstantCE.servicedata_use_survey && meterMap.containsKey(MeterSign)
													&& meterMap.get(MeterSign).getCt() != null) {
												sb.append("servicedata_" + MeterSign + "_" + FunctionTypeId + "*" + meterMap.get(MeterSign).getCt());
											} else {
												sb.append("servicedata_" + MeterSign + "_" + FunctionTypeId);
											}
										}
									}
								}
							}

							finishMap.put(node.getSign(), sb.toString());
						}

					}
				}
				if (!has_change) {
					break;
				}
			}

			String time_begin = sdf.format(sdf_.parse(bcu.getBcuseFrom()));
			String time_end = sdf.format(sdf_.parse(bcu.getBcuseTo()));

			if (ConstantCE.servicedata_use_survey) {
				for (SResearchNode node : ResearchNodeArray) {
					if (node.getMeter() == null) {
						continue;
					}
					if (!childrenMap.containsKey(node.getSign())) {
						continue;
					}
					List<SResearchNode> children = childrenMap.get(node.getSign());
					List<String> meter_children = new ArrayList<String>();
					for (SResearchNode child : children) {
						if (child.getMeter() != null) {
							meter_children.add(child.getMeter());
						}
					}
					if (meter_children.size() > 0) {
						String parentMeterSign = node.getMeter();
						long parentFunctionTypeId = meterMap.get(parentMeterSign).getFuncid();
						if (ConstantWeb.funcid_10101_to_0) {
							if (parentFunctionTypeId == 10101) {
								parentFunctionTypeId = 0;
							}
						}

						StringBuffer sb = new StringBuffer();
						for (String meter_child : meter_children) {
							String MeterSign = meter_child;
							long FunctionTypeId = meterMap.get(MeterSign).getFuncid();
							if (ConstantWeb.funcid_10101_to_0) {
								if (FunctionTypeId == 10101) {
									FunctionTypeId = 0;
								}
							}
							if (!InputMap.containsKey(MeterSign + "_" + FunctionTypeId)) {
								InputMap.put(MeterSign + "_" + FunctionTypeId, true);
							}
							if (sb.length() > 0) {
								sb.append("+");
							}
							sb.append("servicedata_" + MeterSign + "_" + FunctionTypeId);
						}
						String expression = sb.toString();

						Entity_ci_instanceoutput output = new Entity_ci_instanceoutput();
						output.setObject_sign(object_sign);
						output.setInstance_name(instance_name);
						output.setOutput_sign("servicedata_children_" + node.getMeter());
						output.setTime_begin(time_begin);
						output.setTime_end(time_end);
						output.setData_class("consumption");
						output.setDao_type("servicedata");
						JSONObject dao_key = new JSONObject();
						dao_key.put("building", collectBuildingSign);
						dao_key.put("meter", "children_" + parentMeterSign);
						dao_key.put("funcid", parentFunctionTypeId);
						output.setDao_key(dao_key.toJSONString());
						output.setNeed_all_var_data(false);
						output.setTime_period_intialized(true);
						output.setTime_period(time_period);
						output.setMode("expression");
						output.setExpression(expression);
						outputList.add(output);
					}
				}
			}

			for (SResearchNode entity : ResearchNodeArray) {
				if (entity.getIsComputeUnit().equals("Y")) {
					String expression = unit2ExpressionMap.get(entity.getSign());

					Entity_ci_instanceoutput output = new Entity_ci_instanceoutput();
					output.setObject_sign(object_sign);
					output.setInstance_name(instance_name);
					output.setOutput_sign("objectdata_" + entity.getSign());
					output.setTime_begin(time_begin);
					output.setTime_end(time_end);
					output.setData_class("consumption");
					output.setDao_type("objectdata");
					JSONObject dao_key = new JSONObject();
					dao_key.put("building", projectSign);
					dao_key.put("objectSign", entity.getSign());
					output.setDao_key(dao_key.toJSONString());
					output.setNeed_all_var_data(false);
					output.setTime_period_intialized(true);
					output.setTime_period(time_period);
					output.setMode("expression");
					output.setExpression(expression);
					outputList.add(output);
				} else {
					if (ConstantAgent.objectdata_Service) {
						if (entity.getMeter() != null) {
							String MeterSign = entity.getMeter();
							long FunctionTypeId = meterMap.get(MeterSign).getFuncid();
							if (ConstantWeb.funcid_10101_to_0) {
								if (FunctionTypeId == 10101) {
									FunctionTypeId = 0;
								}
							}
							if (!InputMap.containsKey(MeterSign + "_" + FunctionTypeId)) {
								InputMap.put(MeterSign + "_" + FunctionTypeId, true);
							}
							String expression;
							if (!ConstantCE.servicedata_use_survey && meterMap.containsKey(MeterSign) && meterMap.get(MeterSign).getCt() != null) {
								expression = "servicedata_" + MeterSign + "_" + FunctionTypeId + "*" + meterMap.get(MeterSign).getCt();
							} else {
								expression = "servicedata_" + MeterSign + "_" + FunctionTypeId;
							}

							Entity_ci_instanceoutput output = new Entity_ci_instanceoutput();
							output.setObject_sign(object_sign);
							output.setInstance_name(instance_name);
							output.setOutput_sign("objectdata_" + entity.getSign());
							output.setTime_begin(time_begin);
							output.setTime_end(time_end);
							output.setData_class("consumption");
							output.setDao_type("objectdata");
							JSONObject dao_key = new JSONObject();
							dao_key.put("building", building.getBuildingSign());
							dao_key.put("objectSign", entity.getSign());
							output.setDao_key(dao_key.toJSONString());
							output.setNeed_all_var_data(false);
							output.setTime_period_intialized(true);
							output.setTime_period(time_period);
							output.setMode("expression");
							output.setExpression(expression);
							outputList.add(output);
						}
					}
				}
			}
			for (SEnergyModelNode node : EnergyModel.getNodeArray()) {
				String expression = finishMap.get(node.getSign());
				if (expression.length() == 0) {
					continue;
				}

				Entity_ci_instanceoutput output = new Entity_ci_instanceoutput();
				output.setObject_sign(object_sign);
				output.setInstance_name(instance_name);
				output.setOutput_sign("energydata_" + node.getSign());
				output.setTime_begin(time_begin);
				output.setTime_end(time_end);
				output.setData_class("consumption");
				output.setDao_type("energydata");
				JSONObject dao_key = new JSONObject();
				dao_key.put("building", building.getBuildingSign());
				dao_key.put("energyModelSign", bcu.getEnergyModelSign());
				dao_key.put("energyModelNodeSign", node.getSign());
				output.setDao_key(dao_key.toJSONString());
				output.setNeed_all_var_data(false);
				output.setTime_period_intialized(true);
				output.setTime_period(time_period);
				output.setMode("expression");
				output.setExpression(expression);
				outputList.add(output);
			}
		}

		for (String input_sign : InputMap.keySet()) {
			int index_ = input_sign.lastIndexOf('_');
			String meter = input_sign.substring(0, index_);
			long funcid = Long.parseLong(input_sign.substring(index_ + 1));
			Entity_ci_instanceinput input = new Entity_ci_instanceinput();
			input.setObject_sign(object_sign);
			input.setInstance_name(instance_name);
			input.setInput_sign("servicedata_" + input_sign);
			input.setTime_period(time_period);
			input.setData_class("consumption");
			input.setDao_type("servicedata");
			JSONObject dao_key = new JSONObject();
			dao_key.put("building", projectSign.substring("Pj".length()));
			dao_key.put("meter", meter);
			dao_key.put("funcid", funcid);
			input.setDao_key(dao_key.toJSONString());
			input.setSource_object_sign(collectBuildingSign);
			input.setSource_instance_name("servicedata");
			input.setSource_output_sign("servicedata_" + input_sign);
			inputList.add(input);
		}

		instance.setInputs(inputList.toArray(new Entity_ci_instanceinput[0]));
		instance.setOutputs(outputList.toArray(new Entity_ci_instanceoutput[0]));

		return instance;
	}

	private static CommonTreeNodeStream GenerateParser(String expression) throws Exception {
		ANTLRInputStream input = new ANTLRInputStream(new ByteArrayInputStream((expression + "$").getBytes()));
		EMSLexer lexer = new EMSLexer(input);
		CommonTokenStream tokens = new CommonTokenStream(lexer);
		EMSParser parser = new EMSParser(tokens);
		EMSParser.prog_return r = parser.prog();

		CommonTree t = (CommonTree) r.getTree();
		CommonTreeNodeStream nodes = new CommonTreeNodeStream(t);
		nodes.setTokenStream(tokens);

		return nodes;
	}
}
